#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from __future__ import division

import sys
import os

import codecs

import chardet

from PyQt4 import QtGui, QtCore, uic


SCRIPTNAME = ''
try:
    SCRIPTNAME = __file__
except:
    SCRIPTNAME = sys.argv[0]

if sys.platform == 'win32':
    enc = 'mbcs'
else:
    enc = 'utf8'
CWD = unicode(os.path.abspath(os.path.dirname(SCRIPTNAME)), enc)

class ConvertApp(QtGui.QMainWindow):
  ftext = ''

  def __init__(self):
    # QtGui.QMainWindow.__init__(self, None, QtCore.Qt.WindowStaysOnTopHint)
    QtGui.QMainWindow.__init__(self)
    self.setAcceptDrops(True)

    self.ui = uic.loadUi('shiftjis2utf8_text.ui')
    self.ui.setWindowFlags(self.ui.windowFlags() | QtCore.Qt.WindowStaysOnTopHint)
    self.ui.setModal(True)
    self.ui.show()

    self.ui.setAcceptDrops(True)
    self.ui.dragEnterEvent = self.dragEnterEvent
    self.ui.dragMoveEvent = self.dragEnterEvent
    self.ui.dropEvent = self.dropEvent

    self.ui.buttonBox.accepted.connect(self.btnConvert)
    self.ui.buttonBox.clicked.connect(self.btnClick)

    self.ui.lstItems.currentItemChanged.connect(self.currentItemChanged)

    # self.ui.buttonBox.button(QtGui.QDialogButtonBox.Ok).setText(self.tr('Ok'))
    # self.ui.buttonBox.button(QtGui.QDialogButtonBox.Reset).setText(self.tr('Reset'))
    # self.ui.buttonBox.button(QtGui.QDialogButtonBox.Discard).setText(self.tr('Discard'))
    # self.ui.buttonBox.button(QtGui.QDialogButtonBox.Close).setText(self.tr('Close'))
    # self.ui.buttonBox.button(QtGui.QDialogButtonBox.Apply).setText(self.tr('Apply'))

  def dragEnterEvent(self, event):
    if event.mimeData().hasUrls():
      event.accept()
    else:
      event.ignore()

  def dragMoveEvent(self, event):
    if event.mimeData().hasUrls():
      event.accept()
    else:
      event.ignore()

  def dropEvent(self, event):
    idx = self.ui.lstItems.count()

    for url in event.mimeData().urls():
      path = url.toLocalFile().toLocal8Bit().data()
      item = QtGui.QListWidgetItem()
      item.setText(path.decode('mbcs'))
      item.setBackgroundColor(QtGui.QColor(0, 200, 0, 32))
      self.ui.lstItems.addItem(item)

    for idx in xrange(idx, win.ui.lstItems.count()):
      item = win.ui.lstItems.item(idx)
      # win.ui.lstItems.setCurrentItem(item)
      path = unicode(item.text())
      print(path)
      if os.path.isfile(path):
        self.ftext = path
        try:
          result = self.loadFile(path)
          item.setToolTip(result[1])
          item.setWhatsThis(result[1])
          item.setStatusTip(result[1])
          if result[0]:
            item.setBackgroundColor(QtGui.QColor(0, 250, 0, 64))
            self.saveFile(path)
          else:
            item.setBackgroundColor(QtGui.QColor(200, 0, 0, 64))
        except:
          continue

    self.ui.lstItems.setCurrentRow(0)

  def currentItemChanged(self, currentItem, prevItem):
    if currentItem:
      path = unicode(currentItem.text())
      self.loadFile(path)

  def loadFile(self, ftext, autodetect=False):
    with codecs.open(ftext, 'r') as f:
      lines = f.read()

    tip = u'NG'
    result = False
    ftype = chardet.detect(lines)
    if ftype and ('encoding' in ftype):
      self.ui.edUTF8.setPlainText(lines.decode(ftype['encoding']))
      if ftype['encoding'].lower() == 'shift_jis':
        tip = u'OK'
        result = True
      info = u'%s, File Encoding : %s' % (tip, ftype['encoding'])
    else:
      info = u'%s, File Encoding : %s' % (tip, u'UNKNOWN')

    return(result, info)


  def saveFile(self, ftext):
    if os.path.isfile(ftext):
      fn = os.path.splitext(ftext)
      # ftext = fn[0].decode('mbcs') + '_utf8' + fn[1]
      ftext = fn[0] + '_utf8' + fn[1]
      with codecs.open(ftext, 'w') as f:
        f.write(codecs.BOM_UTF8)
        f.write(self.ui.edUTF8.toPlainText())

  def btnConvert(self):
    item = self.ui.lstItems.currentItem()
    if item:
      path = unicode(item.text())
      if os.path.isfile(path):
        self.ftext = path
        self.saveFile(self.ftext)

  def btnClick(self, button):
    btnID = self.ui.buttonBox.standardButton(button)
    # print(hex(btnID))
    if   btnID == 0x00000400:  ## OK
      pass
    elif btnID == 0x00200000:  ## Close
      pass
    elif btnID == 0x00800000:  ## Discard
      self.ui.lstItems.clear()
      pass
    elif btnID == 0x02000000:  ## Apply
      pass
    elif btnID == 0x04000000:  ## Reset
      self.ui.lstItems.clear()
      self.ui.edUTF8.clear()
      pass
    else:
      pass

    pass

if __name__ == "__main__":
  # Create Application
  app = QtGui.QApplication(sys.argv)

  # set app icon
  app_icon = QtGui.QIcon()
  app_icon.addFile('hana.png', QtCore.QSize(16,16))
  app_icon.addFile('hana.png', QtCore.QSize(24,24))
  app_icon.addFile('hana.png', QtCore.QSize(32,32))
  app_icon.addFile('hana.png', QtCore.QSize(48,48))
  app_icon.addFile('hana.png', QtCore.QSize(256,256))
  app.setWindowIcon(app_icon)

  # load zh_CN locale
  locale = QtCore.QLocale.system()

  translator_qt = QtCore.QTranslator()
  folder_qt = unicode(os.path.dirname(QtCore.__file__), enc)
  translator_qt.load(locale, 'qt', prefix='_', directory = os.path.join(folder_qt, 'translations'), suffix='.qm')
  app.installTranslator(translator_qt)

  filename_app = os.path.splitext(os.path.basename(SCRIPTNAME))[0]
  folder_app = os.path.join(CWD, 'i18n')
  translator_app = QtCore.QTranslator()
  translator_app.load(locale, filename_app, prefix='_', directory = folder_app, suffix='.qm')
  app.installTranslator(translator_app)

  # Create Application Window
  win = ConvertApp()

  # Run Application
  sys.exit(app.exec_())
